// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    appSphere.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//           Sebastian Boblest
//
//    Apparent view of a moving sphere with depth information
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++

usepackage("amssymb");
usepackage("mathptmx");

import geometry;
import graph;

import sr_camera;
import sr_apparent;

size(8cm,0);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// observer position, time, field of view
triple xObs = ( 0.0, -10.0, -1.0 );
real   tObs = 6.0; //length(xObs);

real   fovY   = radians(40.0);

// camera: point-of-interest (center)
//         and vertical-up vector
triple center = ( 0.0, 0.0, 0.0 );
triple upV    = ( 0.0, 0.0, 1.0 );

// offset between S and S'
triple aOff   = ( 0.0, 0.0, 0.0 );

// object velocity
triple beta = (0.95,0.0,0.0);

// point in S'
triple XS = (0,0,0);

// direction of line
triple sigma1 = (0,0,1);
triple sigma2 = (1,0,0);
triple sigma3 = (0,-1,0);

real r = 1.0;
real Theta = pi*0.5;
real Phi   = 0.0;

real phistep = 0.01*pi;
real thetastep = 0.005*pi;
real delta = 0.3bp;

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++


// View-Projection matrix
real[][] projMatrix = getPerspMatrix( fovY, 1.0, 0.1, 100.0 );
real[][] viewMatrix = getViewMatrix( xObs, appPointPos(aOff,xObs,beta,XS,tObs), upV );
real[][] MVP = projMatrix*viewMatrix;       

pair appSphereLat( real s ) {
    triple x = appSpherePos( aOff, xObs, beta, XS, sigma1, sigma2, sigma3, r, Theta, s, tObs );
    return map(MVP,x);
}

pair appSphereLon( real s ) {
    triple x = appSpherePos( aOff, xObs, beta, XS, sigma1, sigma2, sigma3, r, s, Phi, tObs );
    return map(MVP,x);
}

triple appSphereLatZ( real s ) {
    triple x = appSpherePos( aOff, xObs, beta, XS, sigma1, sigma2, sigma3, r, Theta, s, tObs );
    return mapWithZ(MVP,x);
}

triple appSphereLonZ( real s ) {
    triple x = appSpherePos( aOff, xObs, beta, XS, sigma1, sigma2, sigma3, r, s, Phi, tObs );
    return mapWithZ(MVP,x);
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++    
       
      
       
// minimal observer canvas       
pair ll = (-1,-1);
pair ur = (1.0,0.5);
fill(ll--(ur.x,ll.y)--ur--(ll.x,ur.y)--cycle,white);
//drawBox(MVP);

real thetaStep = radians(15.0);
real phiStep   = radians(15.0);

//tObs = 4.93;

real wmoving = 0.5bp;
real wrest = 0.5bp;


real minz = 1.0;
real maxz = -1.0;

for(real t=thetaStep; t<pi; t+=thetaStep) {
  Theta = t;
  for( real phi = 0; phi<2.0 * pi; phi +=phistep){
    minz = min(appSphereLatZ(phi).z,minz);
    maxz = max(appSphereLatZ(phi).z,maxz);
  }
}

for(real t=0; t<2.0*pi; t+=phiStep) {
  Phi = t;
  for( real theta = 0; theta<pi-thetastep; theta +=thetastep){
    minz = min(appSphereLonZ(theta).z, minz);
    maxz = max(appSphereLonZ(theta).z, maxz);
 
  }
}

//write("minz = ", minz);
//write("maxz = ", maxz);

//draw picture with correct depth

int pnum = 0;
// determine array size
for(real t=0; t<2.0*pi; t+=phiStep) {
  Phi = t;  
    for( real theta = 0; theta<pi-thetastep; theta +=thetastep){
    ++pnum; 
  }
}

for(real t=thetaStep; t<pi; t+=thetaStep) {
  Theta = t;
  for( real phi = 0; phi<2.0 * pi; phi +=phistep){
    ++pnum;
  }
}
real [][] p = new real[pnum][5];
pnum = 0;

//compute all line segments and sort them according to z
for(real t=0; t<2.0*pi; t+=phiStep) {
  Phi = t;
  
    for( real theta = 0; theta<pi-thetastep; theta +=thetastep){
    p[pnum][0] = appSphereLonZ(theta).z;
    p[pnum][1] = appSphereLonZ(theta).x;
    p[pnum][2] = appSphereLonZ(theta).y;
    p[pnum][3] = appSphereLonZ(theta+thetastep).x;
    p[pnum][4] = appSphereLonZ(theta+thetastep).y;

    ++pnum; 
  }
}

for(real t=thetaStep; t<pi; t+=thetaStep) {
  Theta = t;
  for( real phi = 0; phi<2.0 * pi; phi +=phistep){
    p[pnum][0] = appSphereLatZ(phi).z;
    p[pnum][1] = appSphereLatZ(phi).x;
    p[pnum][2] = appSphereLatZ(phi).y;
    p[pnum][3] = appSphereLatZ(phi+phistep).x;
    p[pnum][4] = appSphereLatZ(phi+phistep).y;
    
    ++pnum;
  }
}
// draw all line segments with low z values first
real [][] psorted = sort(p);
for(int i=pnum-1; i>=0; i=i-1){
 draw((psorted[i][1],psorted[i][2])--(psorted[i][3],psorted[i][4]), colorZ(psorted[i][0], minz, maxz, wmoving) );
}


beta = (0.0,0.0,0.0);

for(int i=4; i<=6; ++i) {
    aOff = (-17.5+2.5*i, 0.0, -2.2 );

    real minz = 1.0;
    real maxz = -1.0;
    
    for(real t=thetaStep; t<pi; t+=thetaStep) {
        Theta = t;
        for( real phi = 0; phi<2.0 * pi; phi +=phistep){
            minz = min(appSphereLatZ(phi).z,minz);
            maxz = max(appSphereLatZ(phi).z,maxz);
        }
    }
    
    for(real t=0; t<2.0*pi; t+=phiStep) {
        Phi = t;
        for( real theta = 0; theta<pi-thetastep; theta +=thetastep){
            minz = min(appSphereLonZ(theta).z, minz);
            maxz = max(appSphereLonZ(theta).z, maxz);     
        }
    }
    
    int pnum = 0;
    // determine array size
    for(real t=0; t<2.0*pi; t+=phiStep) {
        Phi = t;  
            for( real theta = 0; theta<pi-thetastep; theta +=thetastep){
            ++pnum; 
        }
    }
    
    for(real t=thetaStep; t<pi; t+=thetaStep) {
        Theta = t;
        for( real phi = 0; phi<2.0 * pi; phi +=phistep){
            ++pnum;
        }
    }
    real [][] p = new real[pnum][5];
    pnum = 0;
    
    //compute all line segments and sort them according to z
    for(real t=0; t<2.0*pi; t+=phiStep) {
        Phi = t;
      
        for( real theta = 0; theta<pi-thetastep; theta +=thetastep){
            p[pnum][0] = appSphereLonZ(theta).z;
            p[pnum][1] = appSphereLonZ(theta).x;
            p[pnum][2] = appSphereLonZ(theta).y;
            p[pnum][3] = appSphereLonZ(theta+thetastep).x;
            p[pnum][4] = appSphereLonZ(theta+thetastep).y;
        
            ++pnum; 
        }
    }
    
    for(real t=thetaStep; t<pi; t+=thetaStep) {
        Theta = t;
        for( real phi = 0; phi<2.0 * pi; phi +=phistep){
            p[pnum][0] = appSphereLatZ(phi).z;
            p[pnum][1] = appSphereLatZ(phi).x;
            p[pnum][2] = appSphereLatZ(phi).y;
            p[pnum][3] = appSphereLatZ(phi+phistep).x;
            p[pnum][4] = appSphereLatZ(phi+phistep).y;
            
            ++pnum;
        }
    }
    
    // draw all line segments with low z values first
    real [][] psorted = sort(p);
    for(int i=pnum-1; i>=0; i=i-1) {
        //write("z=",psorted[i][0]);
        draw((psorted[i][1],psorted[i][2])--(psorted[i][3],psorted[i][4]), colorZ(psorted[i][0], minz, maxz, wrest) );
    }
}
