// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    sr_apparent.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//
//   Algorithms to determine the apparent position of a
//   point, line, spherical point.
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++


// ---------------------------------------------------------
real calcGamma( triple beta ) {
    return 1.0/sqrt( 1.0 - dot(beta,beta) );
}


// ---------------------------------------------------------
real calcRho( real gam, triple xS, triple beta, real tObs ) {
    return gam * dot(xS,beta) - tObs;
}

// ---------------------------------------------------------
real[] lorentzMatrix( triple beta ) {
    real[] Lambda = new real[16];
    
    real gam = calcGamma(beta);
    real g2  = gam^2/(gam+1);
    
    Lambda[0] = gam;
    Lambda[1] = Lambda[4]  = gam*beta.x;
    Lambda[2] = Lambda[8]  = gam*beta.y;
    Lambda[3] = Lambda[12] = gam*beta.z;
    
    Lambda[5] = 1.0 + g2 * beta.x * beta.x;
    Lambda[6] = g2 * beta.x * beta.y;
    Lambda[7] = g2 * beta.x * beta.z;
    
    Lambda[9]  = g2 * beta.y * beta.x;
    Lambda[10] = 1.0 + g2 * beta.y * beta.y;
    Lambda[11] = g2 * beta.y * beta.z;
    
    Lambda[13] = g2 * beta.z * beta.x;
    Lambda[14] = g2 * beta.z * beta.y;
    Lambda[15] = 1.0 + g2 * beta.z * beta.z;
    
    return Lambda;
}

// ---------------------------------------------------------
real[] poincareTransf( triple beta, real[] xs, real[] aOff ) {
    real[] Lambda = lorentzMatrix(beta);
    real[] newPos = new real[4];
    for(int i=0; i<4; ++i) {
        newPos[i] = 0;
        for(int j=0; j<4; ++j) {
            newPos[i] += Lambda[4*i+j] * (xs[j] + aOff[j]);
        }
    }
    return newPos;
}

// ---------------------------------------------------------
real[] invPoincareTransf( triple beta, real[] x, real[] aOff ) {
    real[] invLambda = lorentzMatrix(-beta);
    real[] newPos = new real[4];
    for(int i=0; i<4; ++i) {
        newPos[i] = 0;
        for(int j=0; j<4; ++j) {
            newPos[i] += invLambda[4*i+j] * (x[j] - aOff[j]);
        }
    }
    return newPos;
}




// ---------------------------------------------------------
//   Apparent position of the point xS
//
//    aOff :  Offset a between S and S'
//    xObs :  Observer position in S
//    beta :  Velocity of S' wrt. S
//    xS   :  Position of point in S'
//    tObs :  Observation time
// ---------------------------------------------------------
triple appPointPos( triple aOff, triple xObs, triple beta, triple xS, real tObs ) {
    
    // gamma factor
    real gam = calcGamma(beta);
    real rho = calcRho(gam,xS,beta,tObs);
    
    triple eta  = xS + gam*gam/(gam+1.0)*dot(xS,beta)*beta + aOff - xObs;
    real   ber  = dot(beta,eta) - rho;
    real omega02 = gam*gam*ber*ber - rho*rho + dot(eta,eta);
     
    // apparent time x'0
    real xs0 = gam*ber - sqrt(omega02);
    
    // apparent position x'
    return gam * xs0 * beta + eta + xObs;
}


// ---------------------------------------------------------
//   Apparent position of points on a line
//
//    aOff   :  Offset a between S and S'
//    xObs   :  Observer position in S
//    beta   :  Velocity of S' wrt. S
//    xS     :  Position of point in S'
//    sigmaS :  Direction of the line in S'
//    s      :  parameter of line in S'
//    tObs   :  Observation time
// ---------------------------------------------------------
triple appLinePos( triple aOff, triple xObs, triple beta, triple xS, triple sigmaS, real s, real tObs ) {

    // gamma factor
    real gam = calcGamma(beta);
    real g2  = gam*gam/(gam+1.0);
    real rho = calcRho(gam,xS,beta,tObs);
    
    triple eta = xS + g2*dot(xS,beta)*beta + aOff - xObs;
    real   rb  = rho-dot(beta,eta);
    
    real omega02 = gam*gam*rb*rb - rho*rho + dot(eta,eta);
    
    real upsilon  = dot(eta,sigmaS) + dot(beta,sigmaS)*(-gam*rho + g2*dot(beta,eta));
    real omegal = sqrt( omega02 + 2*s*upsilon + s*s);
    
    // apparent time x'0
    real   ts  = -gam*rb - omegal;
    
    // apparent position x'
    return gam*ts*beta + eta + xObs + s*sigmaS + g2*s*dot(beta,sigmaS)*beta;    
}



// ---------------------------------------------------------
//   Apparent position of a point on a sphere
//
//    aOff   :  Offset a between S and S'
//    xObs   :  Observer position in S
//    beta   :  Velocity of S' wrt. S
//    xS     :  Position of point in S'
//    sigma1 :  First basis vector in S'
//    sigma2 :  Second basis vector in S'
//    sigma3 :  Third basis vector in S'
//    radius :  Radius of sphere in S'
//    theta  :  colatitude angle in S'
//    phi    :  azimuth angle in S'
//    tObs   :  Observation time
// ---------------------------------------------------------
triple appSpherePos( triple aOff, triple xObs, triple beta, triple xS, 
                     triple sigma1, triple sigma2, triple sigma3, 
                     real radius, real theta, real phi, real tObs ) {

    // gamma factor 
    real gam = calcGamma(beta);
    real g2  = gam*gam/(gam+1.0);
    real rho = calcRho(gam,xS,beta,tObs);
    
    triple eta = xS + g2*dot(xS,beta)*beta + aOff - xObs;
    real   ber = dot(beta,eta) - rho;
            
    real s1 = radius*sin(theta)*cos(phi);
    real s2 = radius*sin(theta)*sin(phi);
    real s3 = radius*cos(theta);
    
    triple sSum = s1 * sigma1 + s2 * sigma2 + s3 * sigma3;
    real   sb   = dot(sSum,beta);
    triple mu   = eta + (-gam*rho + g2*dot(beta,eta)) * beta;
    
    real omega02 = gam*gam * ber*ber - rho*rho + dot(eta,eta);
    
    real omegas = sqrt( omega02 + 2 * dot(mu,sSum) + radius^2);
    
    // apparent time x'0
    real ts  = gam * ber - omegas;
    
    // apparent position x'
    return  gam*ts*beta + eta + xObs + sSum + g2 * sb * beta;        
}


// ---------------------------------------------------------
//
// ---------------------------------------------------------
triple tangentCone( triple xS, triple aOff, triple beta, real rS,
                    real psiS, triple xObs, real tObs ) {
                    
    real[] a = new real[4];
    a[0] = 0.0;
    a[1] = aOff.x;
    a[2] = aOff.y;
    a[3] = aOff.z;
    
    real[] o = new real[4];
    o[0] = tObs;
    o[1] = xObs.x;
    o[2] = xObs.y;
    o[3] = xObs.z;
    
    real[] oS  = invPoincareTransf(beta, o, a);
    real tObsS = oS[0];
    triple xObsS = (oS[1],oS[2],oS[3]);

    triple d = xS - xObsS;
    
    real alphaS = asin(rS / length(d));
    real Rs = rS * cos(alphaS);
    
    triple sigma0 = (0,0,1);
    triple sigma1 = d/length(d);
    triple sigma2 = cross(sigma0,sigma1);
    triple sigma2 = sigma2 / length(sigma2);
    triple sigma3 = cross(sigma1,sigma2);
    
    triple kS = rS*cos(alphaS)/tan(alphaS) * sigma1 + Rs*cos(psiS)*sigma2 + Rs*sin(psiS)*sigma3;
    
    real xs0 = tObsS - length(d)*cos(alphaS);
    
    triple xcS = xObsS + kS;
    
    real[] xcN = new real[4];
    xcN[0] = xs0;
    xcN[1] = xcS.x;
    xcN[2] = xcS.y;
    xcN[3] = xcS.z;
    real[] xc = poincareTransf(beta,xcN,a);
    return (xc[1],xc[2],xc[3]);
}

