// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    sr_camera.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//
//    Camera model
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++

import math;

/** Calculate cross-product of two 3D vectors.
 * @param a   First vector.
 * @param b   Second vector.
 */
triple crossProd( triple a, triple b ) {
    return ( a.y*b.z - a.z*b.y, a.z*b.x - a.x*b.z, a.x*b.y - a.y*b.x );
}

/** Matrix for a translation.
 * @param t  Translation vector.
 */
real[][] translateMatrix( triple t ) {
    real[][] TM = {{1,0,0,t.x},
                   {0,1,0,t.y},
                   {0,0,1,t.z},
                   {0,0,0,1} };
    return TM;
}

/**  Perspective projection matrix taken from OpenGL.
 * @param fovY     Field of View in y-direction (radians).
 * @param aspect   Aspect ratio.
 * @param n        z-near.
 * @param f        z-far.
 */
real[][] getPerspMatrix( real fovY, real aspect, real n, real f ) {
    real fo = cot(fovY*0.5);
    real[][] M = { {fo/aspect, 0.0, 0.0, 0.0}, 
                   {0.0, fo, 0.0, 0.0},
                   {0.0, 0.0, (n+f)/(n-f), -2.0*f*n/(f-n)},
                   {0.0, 0.0, -1.0, 0.0} };
    return M;
}

/** View matrix of the camera.
 * @param eye    Camera position.
 * @param center Camera's viewing point (point-of-interest).
 * @param upV    Camera's up-vector.
 */         
real[][] getViewMatrix( triple eye, triple center, triple upV ) {
    triple F = center - eye;
    F = F / length(F);
    triple U = upV / length(upV);
    triple s = crossProd(F,U);
    s = s / length(s);
    U = crossProd(s,F);    
    
    real[][] viewMatrix = { {1.0, 0.0, 0.0, 0.0},
                            {0.0, 1.0, 0.0, 0.0},
                            {0.0, 0.0, 1.0, 0.0},
                            {0.0, 0.0, 0.0, 1.0} };
                        
    viewMatrix[0][0] = s.x;
    viewMatrix[0][1] = s.y;
    viewMatrix[0][2] = s.z;
    
    viewMatrix[1][0] = U.x;
    viewMatrix[1][1] = U.y;
    viewMatrix[1][2] = U.z;
    
    viewMatrix[2][0] = -F.x;
    viewMatrix[2][1] = -F.y;
    viewMatrix[2][2] = -F.z;
    viewMatrix = viewMatrix*translateMatrix(-eye);
    return viewMatrix;
    //write(viewMatrix);
}

/** Map a 3D-point via View-Projection-Matrix onto screen.
 * @param MVP   View-Projection matrix.
 * @param pIn   Point to be mapped.
 */
pair map( real[][] MVP, triple pIn ) {
    real[] pw = {pIn.x,pIn.y,pIn.z,1.0};
    real[] qw = MVP*pw;
    return (qw[0],qw[1])/qw[3];
}

triple mapWithZ( real[][] MVP, triple pIn ) {
    real[] pw = {pIn.x,pIn.y,pIn.z,1.0};
    real[] qw = MVP*pw;
    //write("mapwithZ: ", qw[2]/qw[3]);
    return (qw[0],qw[1],qw[2])/qw[3];
}

pen colorZ(real z, real minz, real maxz, real basewidth) {
    real znorm = 1.0 - (z-minz)/(maxz-minz);
    real widthfac = 0.8*znorm + 0.2;
    real vr = 0.8*(1-znorm);
    real vg = vr;
    //write("widthfac: ", widthfac);
	pen col = rgb(vr, vg,1.0);
    return col + basewidth * widthfac ;
	
}

