"""
   File:    animDie.py
   Author:  Thomas Mueller, University of Stuttgart, VISUS

   Call from within python:
            execfile("animDie.py")
"""

import platform
import scipy
import math
import matplotlib.pyplot as plt
from   matplotlib import animation
import sr_apparent as sr
import sr_camera   as cam

# offset between S' and S
aOff = [0,0,0]

# observer position
xObs = [ 8.0, 12.0, 2.0]
center = [0,0,0]

# time span for observation
tObs_start = 12.0
tObs_end   = 17.0
tNumSteps  = 300

tObs_step  = (tObs_end - tObs_start)/tNumSteps

# velocity of S' with respect to S
beta = [0,0.9,0]

# Camera parameters
fovY = 32.0
upV  = [0,0,1]

# Construct Model-View-Projection matrix
projMatrix = cam.getPerspMatrix( fovY, 1.0, 0.1, 100.0 )
viewMatrix = cam.getViewMatrix( xObs, center, upV )
MVP = scipy.dot(projMatrix,viewMatrix)

boxVerts = scipy.array([[0,0,0],[1,0,0],[1,1,0],[0,1,0],[0,0,1],[1,0,1],[1,1,1],[0,1,1]])
boxEdges = scipy.array([[0,1],[1,2],[3,0], [4,5],[5,6],[7,4],[0,4],[1,5]])

numBoxEdges = 8

boxFrontVerts = scipy.array([[0,1,0],[1,1,0],[1,1,1],[0,1,1]])
boxFrontEdges = scipy.array([[0,1],[1,2],[2,3],[3,0]])

numBoxFrontEdges = 4

fig = plt.figure( figsize = (13,6), dpi=80 )
ax  = plt.axes(xlim=(-1.0,1.0), ylim=(-0.6,0.4))
ax.set_aspect('equal')
time_text = ax.text(-0.95,-0.55, '') 

# edges of the moving box
box = [[] for i in range(numBoxEdges)]
for i in range(numBoxEdges):
    box[i], = ax.plot([],[],'b')

boxFront = [[] for i in range(numBoxFrontEdges)]
for i in range(numBoxFrontEdges):
    boxFront[i], = ax.plot([],[],'r')

drawlist = ()


def plotLine( a, vel, xS, sigma, smin, smax, t, col ):    
    x = []
    y = []
    for s in scipy.linspace(smin,smax,30):
        pos = sr.appLinePos( a, xObs, vel, xS, sigma, s, t )
        npos = cam.map(MVP,pos)
        x.append(npos[0])
        y.append(npos[1])
        
    plt.plot(x,y,color=col)

    
def plotBox( a, vel, t ):
    for i in range(numBoxEdges):
        xS = 0.5*(boxVerts[boxEdges[i][0]] - scipy.array([0.5,0.5,0.5]))
        sigma = boxVerts[boxEdges[i][1]] - boxVerts[boxEdges[i][0]]
        plotLine(a,vel,xS,sigma,0,0.5,t,'b')

    for i in range(numBoxFrontEdges):
        xS = 0.5*(boxFrontVerts[boxFrontEdges[i][0]] - scipy.array([0.5,0.5,0.5]))
        sigma = boxFrontVerts[boxFrontEdges[i][1]] - boxFrontVerts[boxFrontEdges[i][0]]
        plotLine(a,vel,xS,sigma,0,0.5,t,'r')


def plotBoxRow():
    for n in range(8):
        a = [0,-10+n*2,-0.75]
        plotBox(a,[0,0,0],0)


def init():
    global drawlist    
    for i in range(numBoxEdges):
        box[i].set_data([],[])
        drawlist += (box[i],)
    for i in range(numBoxFrontEdges):
        boxFront[i].set_data([],[])
        drawlist += (boxFront[i],)    
    time_text.set_text('')
    
    drawlist += (time_text,)
    return drawlist
    
    
def animate(i):       
    tObs = tObs_start + tObs_step * i

    for j in range(numBoxEdges):
        xS = 0.5*(boxVerts[boxEdges[j][0]] - scipy.array([0.5,0.5,0.5]))
        sigma = boxVerts[boxEdges[j][1]] - boxVerts[boxEdges[j][0]]
            
        x = []
        y = []
        for s in scipy.linspace(0,0.5,20):
            pos = sr.appLinePos( aOff, xObs, beta, xS, sigma, s, tObs )
            npos = cam.map(MVP,pos)
            x.append(npos[0])
            y.append(npos[1])
            
        box[j].set_data(x,y)

    for j in range(numBoxFrontEdges):
        xS = 0.5*(boxFrontVerts[boxFrontEdges[j][0]] - scipy.array([0.5,0.5,0.5]))
        sigma = boxFrontVerts[boxFrontEdges[j][1]] - boxFrontVerts[boxFrontEdges[j][0]]
            
        x = []
        y = []
        for s in scipy.linspace(0,0.5,20):
            pos = sr.appLinePos( aOff, xObs, beta, xS, sigma, s, tObs )
            npos = cam.map(MVP,pos)
            x.append(npos[0])
            y.append(npos[1])
            
        boxFront[j].set_data(x,y)
    
    time_text.set_text("tObs = {0:5.2f}".format(tObs))    
    return drawlist


plotBoxRow()

doblit = True
if (platform.system()=="Darwin"):
    doblit = False

anim = animation.FuncAnimation( fig, animate, init_func = init, frames = tNumSteps, interval = 1, blit=doblit)
plt.show()

