"""
    File:    animRod.py
    Author:  Thomas Mueller, University of Stuttgart, VISUS

    Call from within python:
            execfile("animRod.py")
            
    The animation might perform better if started from the
    command line:
            python animRod.py
"""

import platform
import scipy
import math
import matplotlib.pyplot as plt
from   matplotlib import animation
import sr_apparent as sr

# velocity of S' with respect to S
beta  = 0.75


# initial time
tObs_start = -1.0
tObs_step  = 0.005
numSteps   = 800

fig = plt.figure( figsize = (10,6), dpi=80 )
ax  = plt.axes( xlim=(-5.5,1.5), ylim=(-2.5,2.5) )
ax.set_aspect('equal')

rod, = ax.plot([],[],color='#000000',lw=2)
circ, = ax.plot([],[],'-',color='b',lw=0.3)
obsp, = ax.plot([],[],'.',color='b',lw=2)
time_text = ax.text(-5.2,-2.3, '')

# plot grid
plt.grid(b=True)

# plot observer
plt.scatter(0.0,0.0,marker = 'D')

pause = False


def curvature( s, t ):
    return sr.appCurvature( [0,0,0], [0,0,0], [beta,0,0], [0,0,0], [0,1,0], s, t )


def centerOfOscCircle( t ):
    gam = 1.0/math.sqrt(1.0-beta**2)
    return gam**2 * (beta * t - math.fabs(t))
    

def onClick(event):
    global pause
    pause ^= True


def init():
    rod.set_data([],[])
    circ.set_data([],[])
    obsp.set_data([],[])
    time_text.set_text('')
    return rod, circ, obsp, time_text


def animate(i):    
    # offset necessary to prevent from hitting the origin
    tObs = tObs_start + tObs_step * i + 1e-5
    x = []
    y = []
    for s in scipy.linspace(-2.0,2.0,50):
        pos = sr.appLinePos( [0,0,0], [0,0,0], [beta,0,0], [0,0,0], [0,1,0], s, tObs )
        x.append( pos[0] );
        y.append( pos[1] );
    
    xc = []
    yc = []
    center = centerOfOscCircle(tObs)
    radius = 1./curvature(0,tObs)
    for phi in scipy.linspace(0.0,2*math.pi,100):        
        xc.append( center + radius*math.cos(phi) )
        yc.append( radius*math.sin(phi) )
        
    if not pause:
        rod.set_data(x,y)
        circ.set_data(xc,yc)
        obsp.set_data(center,0)
        
    time_text.set_text("tObs = {0:5.2f}".format(tObs))    
    return rod, circ, obsp, time_text


fig.canvas.mpl_connect('button_press_event', onClick)

doblit = True
if (platform.system()=="Darwin"):
    doblit = False
    
anim = animation.FuncAnimation( fig, animate, init_func = init, frames = numSteps, interval = 1, blit=doblit)
plt.show()

