"""
    File:    animRodLight.py
    Author:  Thomas Mueller, University of Stuttgart, VISUS

    Call from within python:
            execfile("animRodLight.py")
            
    The animation might perform better if started from the
    command line:
            python animRodLight.py
"""

import platform
import numpy
import scipy
import math
import matplotlib.pyplot as plt
from   matplotlib import animation
import sr_apparent as sr


# velocity of S' with respect to S
beta   = 0.75

# observation time when the observer will see the rod
tObs    = -0.5

# start of animation
t_start = -5.0
t_step  = 0.01
numSteps = 600


# length of rod
rodLength = 4

# number of rays to follow
numRays = 9
rstep = 0.125

eps = 1e-5


fig = plt.figure( figsize = (10,6), dpi=80 )
ax  = plt.axes( xlim=(-5.5,1.5), ylim=(-2.5,2.5) )
ax.set_aspect('equal')


ts = [[] for i in range(numRays)]
rs = [[] for i in range(numRays)]
lr_start = [[] for i in range(numRays)]
lr_dir   = [[] for i in range(numRays)]

drawlist = ()

for i in range(numRays):
    ts[i]  = 0
    rs[i], = ax.plot([],[],'--',color='r',lw=1)
    
appRod, = ax.plot([],[],color='#000000',lw=2)
actRod, = ax.plot([],[],color='#4444ff',lw=2)
time_text = ax.text(-5.3,-2.4, '')

# plot grid
plt.grid(b=True)

# plot observer
plt.scatter(0.0,0.0,marker = 'D')

pause = False



def calcInitTime( l, tobs ):
    w = math.sqrt(beta**2 * tobs**2 + (1-beta**2) * l**2)
    return -( w - tobs )/( 1 - beta**2 )

def onClick(event):
    global pause
    pause ^= True


def init():
    global ts, lr_start, lr_dir, drawlist
    for i in range(numRays):
        ts[i] = calcInitTime( -rodLength*0.5 + rodLength * i * rstep, tObs )
        rs[i].set_data([],[])
        lr_start[i] = scipy.array([ts[i]*beta, -rodLength*0.5 + rodLength * i * rstep])
        lr_dir[i]   = -lr_start[i]/numpy.linalg.norm(lr_start[i]);
        drawlist += rs[i],
    
    appRod.set_data([],[])
    actRod.set_data([],[])
    time_text.set_text('')
    drawlist += appRod, actRod, time_text
    return drawlist


def animate(i):    
    currTime = t_start + t_step * i + eps
    x = []
    y = []
    xt = []
    yt = []
    for s in scipy.linspace(-rodLength*0.5,rodLength*0.5,50):
        pos = sr.appLinePos( [0,0,0], [0,0,0], [beta,0,0], [0,0,0], [0,1,0], s, tObs )
        x.append( pos[0] );
        y.append( pos[1] );
        xt.append( beta*currTime );
        yt.append( s );
           
    
    if not pause:  
        for i in range(numRays):
            rs[i].set_data([],[])
        appRod.set_data([],[])
        actRod.set_data(xt,yt)
        
        for i in range(numRays):
            lr_end = (0,0)
            if (currTime > ts[i]):
                if (currTime < tObs):
                    lr_end = lr_start[i] + (currTime - ts[i]) * lr_dir[i]
                rs[i].set_data([lr_start[i][0],lr_end[0]],[lr_start[i][1],lr_end[1]])    

        
        if (currTime > tObs):
            appRod.set_data(x,y)
        
    time_text.set_text("tObs = {0:5.2f}".format(currTime))    
    return drawlist



fig.canvas.mpl_connect('button_press_event', onClick)

doblit = True
if (platform.system()=="Darwin"):
    doblit = False

anim = animation.FuncAnimation( fig, animate, init_func = init, frames = numSteps, interval = 1, blit=doblit)
plt.show()
