"""
   File:    animRodView.py
   Author:  Thomas Mueller, University of Stuttgart, VISUS

   Call from within python:
            execfile("animRodView.py")
"""

import platform
import numpy
import scipy
import math
import matplotlib.pyplot as plt
from   matplotlib import animation
import sr_apparent as sr
import sr_camera   as cam

# offset between S' and S
aOff = [0,0,0]

# observer position
xObs = [0,0,0]
center = [-1,0,0]

# time span for observation
tObs_start = -1.1
tObs_end   =  0.1
tNumSteps  = 120

tObs_step  = (tObs_end - tObs_start)/tNumSteps

# velocity of S' with respect to S
beta = [0.9,0,0]

# Camera parameters
fovY = 60.0
upV  = [0,0,1]

# Construct Model-View-Projection matrix
projMatrix = cam.getPerspMatrix( fovY, 1.0, 0.1, 100.0 )
viewMatrix = cam.getViewMatrix( xObs, center, upV )
MVP = scipy.dot(projMatrix,viewMatrix)

rodVerts = scipy.array([[-0.1, -1.0, -0.1],[0.1, -1.0, -0.1],[0.1,  1.0, -0.1],[-0.1,  1.0, -0.1],
                        [-0.1, -1.0,  0.1],[0.1, -1.0,  0.1],[0.1,  1.0,  0.1],[-0.1,  1.0,  0.1]])
                        
rodEdges = scipy.array([[0,1],[2,3],[3,0], 
                        [4,5],[6,7],[7,4],
                        [0,4],[3,7]])

edgeLengths = scipy.array([0.2,0.2,2.0,0.2,0.2,2.0,0.2,0.2])

rodFrontEdges = scipy.array([[1,2],[5,6],[1,5],[2,6]])

edgeFrontLengths = scipy.array([2.0,2.0,0.2,0.2])

numRodEdges = 8
numRodFrontEdges = 4


fig = plt.figure( figsize = (12,6), dpi=80 )
ax  = plt.axes(xlim=(-0.8,0.8), ylim=(-0.4,0.4))
ax.set_aspect('equal')
time_text = ax.text(-0.75,-0.35, '') 

drawlist = ()

# edges of the moving rod
rod = [[] for i in range(numRodEdges)]
for i in range(numRodEdges):
    rod[i], = ax.plot([],[],'b')

rodFront = [[] for i in range(numRodFrontEdges)]
for i in range(numRodFrontEdges):
    rodFront[i], = ax.plot([],[],'r')


    
def init():
    global drawlist
    for i in range(numRodEdges):
        rod[i].set_data([],[])
        drawlist += rod[i],
    for i in range(numRodFrontEdges):
        rodFront[i].set_data([],[])
        drawlist += rodFront[i],
    time_text.set_text('')

    drawlist += (time_text,)
    return drawlist
    
    
def animate(i):       
    tObs = tObs_start + tObs_step * i

    for j in range(numRodEdges):
        xS = 0.5*(rodVerts[rodEdges[j][0]] + rodVerts[rodEdges[j][1]])
        sigma = (rodVerts[rodEdges[j][1]] - rodVerts[rodEdges[j][0]])
        sigma = sigma/numpy.linalg.norm(sigma)
                    
        x = []
        y = []
        for s in scipy.linspace(-edgeLengths[j]*0.5,edgeLengths[j]*0.5,50):
            pos = sr.appLinePos( aOff, xObs, beta, xS, sigma, s, tObs )
            npos = cam.map(MVP,pos)
            x.append(npos[0])
            y.append(npos[1])
            
        rod[j].set_data(x,y)

    for j in range(numRodFrontEdges):
        xS = 0.5*(rodVerts[rodFrontEdges[j][0]] + rodVerts[rodFrontEdges[j][1]])
        sigma = (rodVerts[rodFrontEdges[j][1]] - rodVerts[rodFrontEdges[j][0]])
        sigma = sigma/numpy.linalg.norm(sigma)
            
        x = []
        y = []
        for s in scipy.linspace(-edgeFrontLengths[j]*0.5,edgeFrontLengths[j]*0.5,50):
            pos = sr.appLinePos( aOff, xObs, beta, xS, sigma, s, tObs )
            npos = cam.map(MVP,pos)
            x.append(npos[0])
            y.append(npos[1])
            
        rodFront[j].set_data(x,y)
    
    time_text.set_text("tObs = {0:5.2f}".format(tObs))    

    return drawlist

doblit = True
if (platform.system()=="Darwin"):
    doblit = False
    
anim = animation.FuncAnimation( fig, animate, init_func = init, frames = tNumSteps, interval = 1, blit=doblit)
plt.show()

