"""
   File:    sr_apparent.py
   Author:  Thomas Mueller, University of Stuttgart, VISUS

     Algorithms to determine the apparent position of a
     point, line, spherical point.
"""

import numpy
import scipy
import math

def kdelta( a, b ):
    "The Kronecker delta"
    res = 0
    if (a==b):
        res = 1
    return res
    

def lorentzMatrix( beta ):
    """
      Calculate the Lorentz matrix from the velocity beta, where
      beta must be a 3-array, e.g. [0.4,0.1,0.5]
    """
    gamma = 1.0/math.sqrt(1.0-numpy.dot(beta,beta))
    
    Lambda = numpy.identity(4)
    Lambda[0,0] = gamma;
    for i in range(1,4):
        Lambda[0,i] = gamma*beta[i-1]
        Lambda[i,0] = gamma*beta[i-1]
        for j in range(1,4):
            Lambda[i,j] = kdelta(i,j) + gamma**2/(gamma+1)*beta[i-1]*beta[j-1]
            
    return Lambda


def poincareTransf( beta, xs, a = [0,0,0,0] ):
    """
       The Poincare transformation from S' to S is defined by 
       the Lorentz matrix L and the shift vector a. Then, the 
       coordinates x in S follow from:
           x = L*xs + a
    """
    Lambda = lorentzMatrix(scipy.array(beta));
    return scipy.dot(Lambda,scipy.array(xs)) + scipy.array(a)


def invPoincareTransf( beta, x, a = [0,0,0,0] ):
    invLambda = lorentzMatrix(-scipy.array(beta))
    return scipy.dot(invLambda, scipy.array(x) - scipy.array(a))


def calcGamma( beta ):
    """
        Calculate gamma factor:  gamma = 1 / sqrt( 1 - beta^2 )
        
        beta : 3-array
            Velocity of S' with respect to S.
    """
    lbeta = scipy.array(beta)
    return 1/math.sqrt(1 - scipy.dot(beta,beta))
    

    
def calcRho( gam, xS, beta, tObs ):
    """
        Calculate abbreviation parameter rho
    """
    lxS = scipy.array(xS)
    lbeta = scipy.array(beta)
    return gam * scipy.dot(xS,beta) - tObs;
    


def appPointPos( aOff, xObs, beta, xS, tObs ):
    """
        Determine the apparent position of the point xS
        as seen by the observer located at xObs at 
        observation time tObs
        
        aOff : 3-array
            Offset vector between systems S and S'
                
        xObs : 3-array
            Position of observer with respect to S.
                     
        beta : 3-array
            Velocity of S' with respect to S.        
               
        xS : 3-array
            Position of point with respect to S'.
            
        tObs : real
            Observation time with respect to S.
            
        return x : 3-array
            Apparent point x.            
    """
    # If the parameters are not of scipy type...
    laOff = scipy.array(aOff)
    lxObs = scipy.array(xObs)
    lbeta = scipy.array(beta)
    lxS   = scipy.array(xS)
    
    # help parameters
    gam   = calcGamma(beta)
    rho   = calcRho(gam,xS,beta,tObs)
    
    leta  = lxS + gam**2/(gam+1)*scipy.dot(lxS,lbeta) * lbeta + laOff - lxObs        
    ber   = scipy.dot(lbeta,leta) - rho    
    lomega02 = gam**2 * ber**2 - rho**2 + scipy.dot(leta,leta)
    
    # apparent time x'^0
    xs0   = gam * ber - math.sqrt(lomega02)
    
    # apparent position of x'
    return gam*xs0*lbeta + leta + lxObs



def appLinePos( aOff, xObs, beta, xS, sigmaS, s, tObs ):
    """
        Determine the apparent position of a line with
        reference point xS and parameterized by s
        as seen by the observer located at xObs at 
        observation time tObs
        
        aOff : 3-array
            Offset vector between systems S and S'
                
        xObs : 3-array
            Position of observer with respect to S.
                     
        beta : 3-array
            Velocity of S' with respect to S.        
               
        xS : 3-array
            Position of point with respect to S'.
            
        sigmaS : 3-array
            Direction of line with respect to S'.
            
        s : real
            Parameter of line.
            
        tObs : real
            Observation time with respect to S.     
            
        return x : 3-array
            Apparent point x depending on parameter s.
    """
    # If the parameters are not of scipy type...
    laOff = scipy.array(aOff)
    lxObs = scipy.array(xObs)
    lbeta = scipy.array(beta)    
    lxS   = scipy.array(xS)
    lsigmaS = scipy.array(sigmaS)
    
    gam = calcGamma(beta)
    g2  = gam**2 / (gam + 1.0)
    rho = calcRho(gam,xS,beta,tObs)

    leta = xS + g2 * scipy.dot(lxS,beta) * lbeta + laOff - lxObs    
    ber  = scipy.dot(lbeta,leta) - rho
    
    omega02 = gam**2 * ber**2 - rho**2 + scipy.dot(leta,leta)
    
    upsilon   = scipy.dot(leta,lsigmaS) + scipy.dot(lbeta,lsigmaS) * (-gam*rho + g2 * scipy.dot(lbeta,leta))    
    omegal = math.sqrt( omega02 + 2*s*upsilon + s**2 )
    
    #  apparent time x'0
    xs0 = gam * ber - omegal
    
    # apparent position x'
    return gam * xs0 * lbeta + leta + lxObs + s * lsigmaS + g2 * s * scipy.dot(lbeta,lsigmaS) * lbeta;    



def appCurvature( aOff, xObs, beta, xS, sigmaS, s, tObs ):
    """
        Apparent curvature of line
        
        aOff : 3-array
            Offset vector between systems S and S'
                
        xObs : 3-array
            Position of observer with respect to S.
                     
        beta : 3-array
            Velocity of S' with respect to S.        
               
        xS : 3-array
            Position of point with respect to S'.
            
        sigmaS : 3-array
            Direction of line with respect to S'.
            
        s : real
            Parameter of line.
            
        tObs : real
            Observation time with respect to S.
    """
     # If the parameters are not of scipy type...
    laOff = scipy.array(aOff)
    lxObs = scipy.array(xObs)
    lbeta = scipy.array(beta)    
    lxS   = scipy.array(xS)
    lsigmaS = scipy.array(sigmaS)
    

    
    gam = calcGamma(beta)
    g2  = gam**2 / (gam + 1.0)
    b2  = scipy.dot(lbeta,lbeta)
    rho = calcRho(gam,xS,beta,tObs)

    leta = xS + g2 * scipy.dot(lxS,beta) * lbeta + laOff - lxObs    
        
    ber  = scipy.dot(lbeta,leta) - rho
    
    upsilon   = scipy.dot(leta,lsigmaS) + scipy.dot(lbeta,lsigmaS) * (-gam*rho + g2 * scipy.dot(lbeta,leta))
    
    omega02 = gam**2 * ber**2 - rho**2 + scipy.dot(leta,leta)
    
    omegal = math.sqrt( omega02 + 2*s*upsilon + s**2 )
    
    bs = scipy.dot(lbeta,lsigmaS)
    
    f = math.sqrt( gam**2 * (s+upsilon)**2 / (omegal**2) * b2 - 2.0*gam**2 * (s+upsilon)/omegal * bs + 1 + gam**2*bs**2 )
        
    k1 = ((omegal**2 - (s+upsilon)**2)/(f**3 * omegal**3))**2
    w1 = ((s+upsilon)/omegal)**2 * gam**2 * b2 * (b2 - bs**2)
    w2 = -2*gam**2 * bs * (s+upsilon)/omegal * (b2 - bs**2)
    w3 = b2 + (gam**2 - 2) * bs**2 - gam**2 * bs**4
    
    kap2 = k1 * gam**2 * (w1 + w2 + w3)
    return math.sqrt(kap2)



def appSpherePos( aOff, xObs, beta, xS, sigma1, sigma2, sigma3, rS, thetaS, phiS, tObs ):
    """
        Determine the apparent shape of a sphere in S' with 
        centet point xS, radius rS as seen by the observer located at xObs at 
        observation time tObs
        
        aOff : 3-array
            Offset vector between systems S and S'
                  
        xObs : 3-array
            Position of observer with respect to S.
             
        beta : 3-array
            Velocity of S' with respect to S.
            
        xS : 3-array
            Position of point with respect to S'.
         
        sigma1: 3-array
            First basis vector.
            
        sigma2: 3-array
            Second basis vector.
            
        sigma3: 3-array
            Third basis vector.
            
        rS : real
            Radius of the sphere with respect to S'.
            
        thetaS: real
            Colatitude angle (radians) of sphere point.
            
        phiS: real
            Azimuth angle (radians) of sphere point.
            
        tObs : real
            Observation time with respect to S.
            
        return x : 3-array
            Apparent point depending on (thetaS,phiS).
    """
    # If the parameters are not of scipy type...
    lsigma1 = scipy.array(sigma1)
    lsigma2 = scipy.array(sigma2)
    lsigma3 = scipy.array(sigma3)
        
    laOff  = scipy.array(aOff)
    lxS   = scipy.array(xS)
    lbeta = scipy.array(beta)
    lxObs = scipy.array(xObs)
    
    gam = calcGamma(beta)
    g2  = gam**2 / (gam + 1.0)
    rho = calcRho(gam,xS,beta,tObs)
    b_xS = scipy.dot(xS,beta)
        
    leta = lxS + g2 * b_xS * lbeta + laOff - lxObs
    ber  =  scipy.dot(lbeta,leta) - rho
            
    s1 = rS * math.sin(thetaS) * math.cos(phiS)
    s2 = rS * math.sin(thetaS) * math.sin(phiS)
    s3 = rS * math.cos(thetaS)
            
    lsSum = s1 * lsigma1 + s2 * lsigma2 + s3 * lsigma3
    lsb = scipy.dot( lsSum, lbeta )        
    lmu = leta + ( -gam*rho + g2 * scipy.dot(lbeta,leta) ) * lbeta
    
    omega0_2 = gam**2 * ber**2 - rho**2 + scipy.dot(leta,leta)
    
    omegas = math.sqrt( omega0_2 + 2.0 * scipy.dot(lmu,lsSum) + rS**2 )
    
    # apparent time x'^0
    xs0 = gam * ber - omegas
    
    # apparent position x'
    return gam * xs0 * lbeta + leta + lxObs + lsSum + g2 * lsb * lbeta




def constrEq( aOff, xObs, beta, xS, sigma1, sigma2, sigma3, rS, thetaS, phiS, tObs ):
    # If the parameters are not of scipy type...
    lsigma1 = scipy.array(sigma1)
    lsigma2 = scipy.array(sigma2)
    lsigma3 = scipy.array(sigma3)
        
    laOff  = scipy.array(aOff)
    lxS   = scipy.array(xS)
    lbeta = scipy.array(beta)
    lxObs = scipy.array(xObs)
    
    gam = calcGamma(beta)
    g2  = gam**2 / (gam + 1.0)
    rho = calcRho(gam,xS,beta,tObs)
    b_xS = scipy.dot(xS,beta)
        
    leta = lxS + g2 * b_xS * lbeta + laOff - lxObs
    ber  =  scipy.dot(lbeta,leta) - rho
            
    s1 = rS * math.sin(thetaS) * math.cos(phiS)
    s2 = rS * math.sin(thetaS) * math.sin(phiS)
    s3 = rS * math.cos(thetaS)
    
    ds1dtheta =  rS * math.cos(thetaS) * math.cos(phiS)    
    ds2dtheta =  rS * math.cos(thetaS) * math.sin(phiS)
    ds3dtheta = -rS * math.sin(thetaS) 
    
    ds1dphi   = -rS * math.sin(thetaS) * math.sin(phiS)
    ds2dphi   =  rS * math.sin(thetaS) * math.cos(phiS)
    ds3dphi   = 0.0
            
    lsSum = s1 * lsigma1 + s2 * lsigma2 + s3 * lsigma3
    lsb = scipy.dot( lsSum, lbeta )        
    lmu = leta + ( -gam*rho + g2 * scipy.dot(lbeta,leta) ) * lbeta
    
    omega0_2 = gam**2 * ber**2 - rho**2 + scipy.dot(leta,leta)
    omegas = math.sqrt( omega0_2 + 2.0 * scipy.dot(lmu,lsSum) + rS**2 )
    
    k = -gam/omegas * lmu + g2*lbeta;
    
    f = scipy.dot( ds1dtheta * lsigma1 + ds2dtheta * lsigma2 + ds3dtheta * lsigma3, k )
    g = scipy.dot( ds1dphi * lsigma1 + ds2dphi * lsigma2, k )
    
    h1 = f * ds1dphi - g * ds1dtheta
    h2 = f * ds2dphi - g * ds2dtheta
    h3 = f * ds3dphi - g * ds3dtheta
    
    q3 = ds1dtheta * ds2dphi - ds2dtheta * ds1dphi
    q1 = ds2dtheta * ds3dphi - ds3dtheta * ds2dphi
    q2 = ds3dtheta * ds1dphi - ds1dtheta * ds3dphi
    
    n = h1 * scipy.cross(lbeta,lsigma1) + q1*lsigma1 + h2 * scipy.cross(lbeta,lsigma2) + q2*lsigma2 + h3 * scipy.cross(lbeta,lsigma3) + q3*lsigma3

    xs0 = gam * ber - omegas
    xs =  gam * xs0 * lbeta + leta + lxObs + lsSum + g2 * lsb * lbeta

    return scipy.dot(n, xs - lxObs)



def tangentCone( xS, aOff, beta, rS, psiS, xObs, tObs ):
    laOff = scipy.array(aOff)
    lxObs = scipy.array(xObs)
    lxS  = scipy.array(xS)
    lbeta = scipy.array(beta)
    
    oS = invPoincareTransf(beta,[tObs,xObs[0],xObs[1],xObs[2]],[0,aOff[0],aOff[1],aOff[2]])

    tObsS = oS[0]
    xObsS = scipy.array([oS[1],oS[2],oS[3]])
    
    d = lxS - xObsS
    #print rS, numpy.linalg.norm(d)

    alphaS = math.asin(rS/numpy.linalg.norm(d))    
    Rs = rS * math.cos(alphaS)
    
    sigma0 = scipy.array([0,0,1])
    sigma1 = d/numpy.linalg.norm(d)
    sigma2 = scipy.cross(sigma0,sigma1)
    sigma2 = sigma2 / numpy.linalg.norm(sigma2)
    sigma3 = scipy.cross(sigma1,sigma2)
    
    kS = rS * math.cos(alphaS) / math.tan(alphaS) * sigma1 + Rs * math.cos(psiS) * sigma2 + Rs * math.sin(psiS) * sigma3
    
    xs0 = tObsS - numpy.linalg.norm(d) * math.cos(alphaS)
    xcS = xObsS + kS
    
    xcN = [xs0, xcS[0],xcS[1],xcS[2]]
    xc = poincareTransf(beta,xcN,[0,aOff[0],aOff[1],aOff[2]])
    return xc
