"""
   File:    sr_camera.py
   Author:  Thomas Mueller, University of Stuttgart, VISUS

     Algorithms to determine the apparent position of a
     point, line, spherical point.
"""

import scipy
import math

def getTranslateMatrix( t ):
    """
        Construct a 4x4 translation matrix.
       
        t : 3-array
            Translation vector.
    """
    lt = scipy.array(t)
    
    mat = scipy.identity(4)
    mat[:,3] = [t[0],t[1],t[2],1]
    return mat


def getPerspMatrix( fovY, aspect, n, f ):
    """
        Construct 4x4 perspective matrix.
        
        fovY : real
            Vertical field of view in degree
            
        aspect : real
            Aspect ratio
            
        n : real
            Near clipping plane.
            
        f : real
            Far clipping plane.
    """
    fo = 1./math.tan(0.5 * math.radians(fovY))
    
    mat = scipy.zeros((4,4))
    mat[0,0] = fo/aspect
    mat[1,1] = fo
    mat[2,2] = (n+f)/(n-f)
    mat[2,3] = -2.*f*n/(f-n)
    mat[3,2] = -1.
    
    return mat


def getViewMatrix( eye, center, upV ):
    """
        Construct view matrix of a camera.
        
        eye : 3-array
            Camera position.
            
        center : 3-array
            Camera's viewing point (point-of-interest).
            
        upV : 3-array
            Camera's up-vector.
    """
    leye = scipy.array(eye)
    lcenter = scipy.array(center)
    lupV = scipy.array(upV)
    
    ez = - (lcenter - leye)
    ez = ez / math.sqrt(scipy.dot(ez,ez))
    U = lupV / math.sqrt(scipy.dot(lupV,lupV))
    
    ex = scipy.cross(U,ez)
    ex = ex / math.sqrt(scipy.dot(ex,ex))
    ey = scipy.cross(ez,ex)
    
    mat = scipy.identity(4)
    mat[0,0:3] = ex
    mat[1,0:3] = ey
    mat[2,0:3] = ez
    
    # matrix multiplication for arrays:
    mat = scipy.dot(mat,getTranslateMatrix(-leye))
    return mat
 
 
def map( MVP, pIn):
    """
    
    """
    lp = scipy.array([pIn[0],pIn[1],pIn[2],1.])
    lq = scipy.dot(MVP,lp)
    return scipy.array([lq[0],lq[1]])/lq[3]

 
def mapZ( MVP, pIn):
    """
    
    """
    lp = scipy.array([pIn[0],pIn[1],pIn[2],1.])
    lq = scipy.dot(MVP,lp)
    return scipy.array([lq[0],lq[1],lq[2]])/lq[3]
